@@ -381,10 +381,18 @@ static void shuffle(size_t keys[], size_t n)
         u = drand48();
         ASSERT(u >= 0.0 && u < 1.0);
 
+        // stev: we have that:
+        // 0 <= u < 1 <=> 0 <= (j + 1) * u < j + 1
+        //            <=> 0 <= floor((j + 1) * u) < j + 1
+        //            <=> 0 <= floor((j + 1) * u) <= j
+        // stev: above we used the following:
+        //   for all x in R and n in Z:
+        //   x < n + 1 <=> floor(x) < n + 1
+
         // stev: make 'k' be a random integer,
-        // between 0 and 'j - 1'
-        k = SIZE_MUL_DOUBLE(j, u);
-        VERIFY(k < j);
+        // between 0 and 'j'
+        k = SIZE_MUL_DOUBLE(j + 1, u);
+        VERIFY(k <= j);
 
         // stev: swap 'keys[k]' and 'keys[j]'
         t = keys[k];
